@@ -35,8 +35,8 @@
 
 #include <pspell/pspell.h>
 
+#include <glib/gi18n.h>
 #include <glib/gstrfuncs.h>
-#include <bonobo/bonobo-i18n.h>
 
 #include "gedit-spell-checker.h"
 
@@ -74,33 +74,86 @@ enum {
 	LAST_SIGNAL
 };
 
-#define KNOWN_LANGUAGES 23
-static GeditLanguage known_languages [KNOWN_LANGUAGES + 1] = 
+static GeditLanguage known_languages [] = 
 {
+	{"af", N_("Afrikaans")},
+	{"am", N_("Amharic")},
+	{"az", N_("Azerbaijani")},
+	{"be", N_("Belarusian")},
+	{"bg", N_("Bulgarian")},
+	{"bn", N_("Bengali")},
 	{"br", N_("Breton")},
 	{"ca", N_("Catalan")},
 	{"cs", N_("Czech")},
+	{"cy", N_("Welsh")},
 	{"da", N_("Danish")},
-	{"de_de", N_("German (Germany)")},
-	{"de_ch", N_("German (Swiss)")},
-	{"en_us", N_("English (American)")},
-	{"en_gb", N_("English (British)")},
-	{"en_ca", N_("English (Canadian)")},
+	{"de_AT", N_("German (Austria)")},
+	{"de_DE", N_("German (Germany)")},
+	{"de_CH", N_("German (Swiss)")},
+	{"el", N_("Greek")},
+	{"en_US", N_("English (American)")},
+	{"en_GB", N_("English (British)")},
+	{"en_CA", N_("English (Canadian)")},
 	{"eo", N_("Esperanto")},
 	{"es", N_("Spanish")},
+	{"et", N_("Estonian")},
+	{"fa", N_("Persian")},	
+	{"fi", N_("Finnish")},
 	{"fo", N_("Faroese")},
-	{"fr_fr", N_("French (France)")},
-	{"fr_ch", N_("French (Swiss)")},
+	{"fr_FR", N_("French (France)")},
+	{"fr_CH", N_("French (Swiss)")},
+	{"ga", N_("Irish")},
+	{"gd", N_("Scottish Gaelic")},
+	{"gl", N_("Gallegan")},
+	{"gv", N_("Manx Gaelic")},
+	{"he", N_("Hebrew")},
+	{"hi", N_("Hindi")},
+	{"hr", N_("Croatian")},
+	{"hsb", N_("Upper Sorbian")},
+	{"hu", N_("Hungarian")},
+	{"ia", N_("Interlingua (IALA)")},
+	{"id", N_("Indonesian")},
+	{"is", N_("Icelandic")},
 	{"it", N_("Italian")},
+	{"ku", N_("Kurdish")},
+	{"la", N_("Latin")},
+	{"lt", N_("Lithuanian")},
+	{"lv", N_("Latvian")},
+	{"mg", N_("Malagasy")},
+	{"mi", N_("Maori")},
+	{"mk", N_("Macedonian")},
+	{"mn", N_("Mongolian")},
+	{"ms", N_("Malay")},
+	{"mt", N_("Maltese")},
+	{"nb", N_("Norwegian Bokmal")},
 	{"nl", N_("Dutch")},
+	{"nn", N_("Norwegian Nynorsk")},
 	{"no", N_("Norwegian")},
+	{"ny", N_("Nyanja")},
 	{"pl", N_("Polish")},
-	{"pt_pt", N_("Portuguese (Portugal)")},
-	{"pt_br", N_("Portuguese (Brazilian)")},
+	{"pt_PT", N_("Portuguese (Portugal)")},
+	{"pt_BR", N_("Portuguese (Brazilian)")},
+	{"qu", N_("Quechua")},
+	{"ro", N_("Romanian")},
 	{"ru", N_("Russian")},
+	{"rw", N_("Kinyarwanda")},
+	{"sc", N_("Sardinian")},
+	{"sk", N_("Slovak")},
+	{"sl", N_("Slovenian")},
 	{"sv", N_("Swedish")},
+	{"sw", N_("Swahili")},
+	{"ta", N_("Tamil")},
+	{"tet", N_("Tetum")},
+	{"tl", N_("Tagalog")},
+	{"tn", N_("Tswana")},
+	{"tr", N_("Turkish")},
 	{"uk", N_("Ukrainian")},
-	{NULL, NULL}
+	{"uz", N_("Uzbek")},
+	{"vi", N_("Vietnamese")},
+	{"wa", N_("Walloon")},
+	{"yi", N_("Yiddish")},
+	{"zu", N_("Zulu")},
+ 	{NULL, NULL}
 };
 
 static void	gedit_spell_checker_class_init 		(GeditSpellCheckerClass * klass);
@@ -272,7 +325,6 @@ gedit_spell_checker_class_init (GeditSpellCheckerClass * klass)
 			  0); 
 }
 
-
 static void
 gedit_spell_checker_init (GeditSpellChecker *spell_checker)
 {
@@ -293,7 +345,6 @@ gedit_spell_checker_new	(void)
 	return spell;
 }
 
-
 static void 
 gedit_spell_checker_finalize (GObject *object)
 {
@@ -307,22 +358,40 @@ gedit_spell_checker_finalize (GObject *object)
 		delete_pspell_manager (spell_checker->manager);
 }
 
-static const GeditLanguage*
+static const GeditLanguage *
 get_language_from_abrev (const gchar *abrev)
 {
-	int i;
-	
+	const GSList *langs;
+
 	g_return_val_if_fail (abrev != NULL, NULL);
-	
-	for (i = 0; known_languages [i].abrev; i++) 
+
+	langs = gedit_spell_checker_get_available_languages ();
+
+	while (langs != NULL)
 	{
-		if (g_ascii_strncasecmp (abrev, known_languages [i].abrev, strlen(known_languages [i].abrev)) == 0)
-			return &known_languages [i];
+		const GeditLanguage *l = (const GeditLanguage *)langs->data;
+
+		if (g_ascii_strncasecmp (abrev, l->abrev, strlen(l->abrev)) == 0)
+			return l;
+
+		langs = g_slist_next (langs);
 	}
 
 	return NULL;
 }
 
+gchar *
+gedit_language_to_key (const GeditLanguage *lang)
+{
+	return g_strdup (lang->abrev);
+}
+
+const GeditLanguage *
+gedit_language_from_key	(const gchar *key)
+{
+	return get_language_from_abrev (key);
+}
+
 static gboolean
 lazy_init (GeditSpellChecker *spell, const GeditLanguage *language, GError **error) 
 {
@@ -337,15 +406,23 @@ lazy_init (GeditSpellChecker *spell, const GeditLanguage *language, GError **err
 	config = new_pspell_config ();
 	g_return_val_if_fail (config != NULL, FALSE);
 	
-	if (language != NULL)
-		pspell_config_replace (config, "language-tag", language->abrev);
-		
+/*
+	g_print ("lazy_init: Language: %s\n", gedit_language_to_string (language));
+*/
 	pspell_config_replace (config, "encoding", "utf-8");
 	
 	pspell_config_replace (config, "mode", "url");
 
-	if (language == NULL)
+	if (language != NULL)
 	{
+		if (get_language_from_abrev (language->abrev) != NULL)
+		{
+			spell->active_lang = language;
+		}
+	}
+	else	
+	{
+		/* First try to get a default language */	
 		const gchar *language_tag;
 
 		language_tag = pspell_config_retrieve (config, "language-tag");
@@ -355,17 +432,31 @@ lazy_init (GeditSpellChecker *spell, const GeditLanguage *language, GError **err
 			spell->active_lang = get_language_from_abrev (language_tag);
 		}
 
-		/*
+/*
 		g_print ("Language tag: %s\n", language_tag);
-		*/
+*/
+	}	
+
+	/* Second try to get a default language */
+
+	if (spell->active_lang == NULL)
+		spell->active_lang = get_language_from_abrev ("en_us");
+
+	/* Last try to get a default language */
+	if (spell->active_lang == NULL)
+	{
+		const GSList *langs;
+		langs = gedit_spell_checker_get_available_languages ();
+		spell->active_lang = (const GeditLanguage *)langs->data;
 	}
-	else
-		spell->active_lang = language;
 
-	/*
+	if (spell->active_lang != NULL)
+	{
+		pspell_config_replace (config, "language-tag", spell->active_lang->abrev);
+	}
+/*
 	g_print ("Language: %s\n", gedit_language_to_string (spell->active_lang ));
-	*/
-
+*/	
 	err = new_pspell_manager (config);
 
 	delete_pspell_config (config);
